import os
import datetime
import json
import ipaddress
import urllib
from flask import Flask, url_for, redirect, \
    render_template, session, request, send_from_directory, abort
from flask_login import LoginManager, login_required, login_user, \
    logout_user, current_user, UserMixin
from flask_sqlalchemy import SQLAlchemy
from requests_oauthlib import OAuth2Session
from requests.exceptions import HTTPError

from flask_auth.config import *
from flask_auth import app, db, User, login_manager


def get_google_auth(state=None, token=None):
    if token:
        return OAuth2Session(Auth.CLIENT_ID, token=token)
    if state:
        return OAuth2Session(
            Auth.CLIENT_ID,
            state=state,
            redirect_uri='http://' + str(request.host) + Auth.REDIRECT_URI)
    oauth = OAuth2Session(
        Auth.CLIENT_ID,
        redirect_uri='http://' + str(request.host) + Auth.REDIRECT_URI,
        scope=Auth.SCOPE)
    return oauth


def request_path(path):
    if path is None:
        return redirect(url_for('login'))
    if 'de-container-cloud' in path:
        return redirect(url_for('route_de_container_cloud_latest'))
    if 'mcp' in path:
        return redirect(url_for('route_mcp_q4_18'))
    if 'mos' in path:
        return redirect(url_for('route_mosk_latest'))
    return redirect(url_for('route_mcp_q4_18'))


def check_user_ip():
    for proxy in TRUSTED_PROXIES:
        if (ipaddress.ip_address(unicode(request.remote_addr)) in
                ipaddress.ip_network(unicode(proxy))):
            return True
    return False

ABS_SITE_ROOT_PATH = '/var/www/'
MIRANTIS_ERROR_PAGES_PATH = '/usr/share/error_pages/mirantis/'


@app.route('/')
def route_main():
    return redirect(url_for('route_welcome'))


@app.route('/welcome/getting-started/')
@app.route('/welcome/mirantis-openstack-for-kubernetes/')
@app.route('/welcome/mirantis-secure-registry/')
@app.route('/welcome/mirantis-container-runtime/')
@app.route('/welcome/mirantis-kubernetes-engine/')
@app.route('/welcome/mirantis-container-cloud/')
def route_default():
   return send_from_directory(ABS_SITE_ROOT_PATH + 'welcome/', 'index.html')


@app.route('/robots.txt')
def static_from_root():
    return send_from_directory(ABS_SITE_ROOT_PATH, request.path[1:])


@app.errorhandler(404)
def page_not_found(e):
    return send_from_directory(MIRANTIS_ERROR_PAGES_PATH, '404.html'), 404


@app.errorhandler(403)
def page_not_found(e):
    return send_from_directory(MIRANTIS_ERROR_PAGES_PATH, '403.html')


@app.errorhandler(500)
def page_not_found(e):
    return send_from_directory(MIRANTIS_ERROR_PAGES_PATH, '5xx.html')


@app.route('/internal/', defaults={'path': ''})
@app.route('/internal/<path:path>')
def route_internal(path):
    if path == '':
        path = 'index.html'

    if (not check_user_ip()) and (not current_user.is_authenticated()):
        return redirect(url_for('login') +
                        '?next=' +
                        urllib.quote_plus("/internal/"+str(path))
                        )

    try:
        return send_from_directory(
            ABS_SITE_ROOT_PATH + 'internal/', str(path)
        )
    except:
        abort(404)


@app.route('/login')
def login():
    msg = request.args.get('msg')
    if msg is None:
        msg = ''

    if current_user.is_authenticated() or check_user_ip():
        return redirect(url_for('route_mcp_master'))
    google = get_google_auth()
    auth_url, state = google.authorization_url(
        Auth.AUTH_URI,
        access_type='online',
        hd='mirantis.com'
    )
    session['oauth_state'] = state
    session['next_step'] = request.args.get('next')
    return render_template('login.html', auth_url=auth_url, msg=msg)


@app.route('/oauth2callback')
def callback():
    next = session.get('next_step')
    if next is None:
        next = url_for('route_mcp_master')

    if (current_user is not None and
            current_user.is_authenticated()) or check_user_ip():
        return redirect(url_for('route_mcp_master'))
    if 'error' in request.args:
        if request.args.get('error') == 'access_denied':
            return redirect(url_for('login', msg='Access denied.'))
        return redirect(url_for('login', msg='Error encountered.'))
    if 'code' not in request.args and 'state' not in request.args:
        return redirect(url_for('login'))
    else:
        google = get_google_auth(state=session.get(u'oauth_state'))
        try:
            token = google.fetch_token(
                Auth.TOKEN_URI,
                client_secret=Auth.CLIENT_SECRET,
                authorization_response=request.url)
        except HTTPError:
            return 'HTTPError occurred.'
        google = get_google_auth(token=token)
        resp = google.get(Auth.USER_INFO)
        if resp.status_code == 200:
            user_data = resp.json()
            email = user_data['email']
            if '@mirantis.com' in email:
                user = User.query.filter_by(email=email).first()
                if user is None:
                    user = User()
                    user.email = email
                user.name = user_data['name']
                print(token)
                user.tokens = json.dumps(token)
                user.avatar = user_data['picture']
                db.session.add(user)
                db.session.commit()
                login_user(user)
                return redirect(next)
            else:
                return redirect(url_for('login', msg='Access denied.'))
        return redirect(url_for('login',
                                msg='Could not fetch your information.'))


@app.route('/logout')
#@login_required
def logout():
    logout_user()
    return request_path(request.referrer)
